@@ -2,14 +2,16 @@
  * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
+ *
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ *
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- * 
+ *
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -17,7 +19,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- * 
+ *
  * @APPLE_LICENSE_HEADER_END@
  */
 #include <libkern/OSByteOrder.h>
@@ -25,6 +27,7 @@
 #include <IOKit/assert.h>
 #include <IOKit/IOLib.h>
 #include <IOKit/IOMessage.h>
+#include <IOKit/IOKitKeys.h>
 
 #include <IOKit/usb/IOUSBController.h>
 #include <IOKit/usb/IOUSBControllerV2.h>
@@ -34,6 +37,14 @@
 
 #define super IOUserClient
 
+#ifndef kIOUserClientCrossEndianKey
+#define kIOUserClientCrossEndianKey "IOUserClientCrossEndian"
+#endif
+
+#ifndef kIOUserClientCrossEndianCompatibleKey
+#define kIOUserClientCrossEndianCompatibleKey "IOUserClientCrossEndianCompatible"
+#endif
+
 
 //================================================================================================
 //	IOKit stuff and Constants
@@ -164,9 +175,12 @@ IOUSBControllerUserClient::getTargetAndMethodForIndex(IOService **target, UInt32
 bool
 IOUSBControllerUserClient::initWithTask(task_t owningTask, void *security_id, UInt32 type, OSDictionary * properties)
 {
-    IOLog("IOUSBControllerUserClient::initWithTask(type %ld)\n", type);
-
-    if (!owningTask)
+	if ( properties != NULL )
+	{
+		properties->setObject( kIOUserClientCrossEndianCompatibleKey, kOSBooleanTrue);
+	}
+		
+	if (!owningTask)
         return false;
 
     fTask = owningTask;
@@ -185,7 +199,7 @@ bool
 IOUSBControllerUserClient::start( IOService * provider )
 {
     fOwner = OSDynamicCast(IOUSBController, provider);
-    IOLog("+IOUSBControllerUserClient::start (%p)\n", fOwner);
+
     if (!fOwner)
         return false;
 
@@ -223,7 +237,6 @@ IOUSBControllerUserClient::open(bool seize)
 IOReturn
 IOUSBControllerUserClient::close()
 {
-    IOLog("+IOUSBControllerUserClient::close\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -236,7 +249,6 @@ IOUSBControllerUserClient::close()
 IOReturn
 IOUSBControllerUserClient::EnableKernelLogger(bool enable)
 {
-    IOLog("+IOUSBControllerUserClient::EnableKernelLogger\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -248,7 +260,6 @@ IOUSBControllerUserClient::EnableKernelLogger(bool enable)
 IOReturn
 IOUSBControllerUserClient::SetDebuggingLevel(KernelDebugLevel inLevel)
 {
-    IOLog("+IOUSBControllerUserClient::SetDebuggingLevel\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -260,7 +271,6 @@ IOUSBControllerUserClient::SetDebuggingLevel(KernelDebugLevel inLevel)
 IOReturn
 IOUSBControllerUserClient::SetDebuggingType(KernelDebuggingOutputType inType)
 {
-    IOLog("+IOUSBControllerUserClient::SetDebuggingType\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -272,7 +282,6 @@ IOUSBControllerUserClient::SetDebuggingType(KernelDebuggingOutputType inType)
 IOReturn
 IOUSBControllerUserClient::GetDebuggingLevel(KernelDebugLevel * inLevel)
 {
-    IOLog("+IOUSBControllerUserClient::GetDebuggingLevel\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -284,7 +293,6 @@ IOUSBControllerUserClient::GetDebuggingLevel(KernelDebugLevel * inLevel)
 IOReturn
 IOUSBControllerUserClient::GetDebuggingType(KernelDebuggingOutputType * inType)
 {
-    IOLog("+IOUSBControllerUserClient::GetDebuggingLevel\n");
     if (!fOwner)
         return kIOReturnNotAttached;
 
@@ -300,7 +308,6 @@ IOUSBControllerUserClient::SetTestMode(UInt32 mode, UInt32 port)
     // this method only available for v2 controllers
     IOUSBControllerV2	*v2 = OSDynamicCast(IOUSBControllerV2, fOwner);
 
-    IOLog("+IOUSBControllerUserClient::SetTestMode");
     if (!v2)
         return kIOReturnNotAttached;
 
@@ -327,19 +334,19 @@ IOUSBControllerUserClient::ReadRegister(UInt32 offset, UInt32 size, void *value)
         case 8:
             bVal = *((UInt8 *)fMemMap->getVirtualAddress() + offset);
             *(UInt8*)value = bVal;
-            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got byte value %p", bVal);
+            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got byte value 0x%x", bVal);
             break;
 
         case 16:
             wVal = OSReadLittleInt16((void*)(fMemMap->getVirtualAddress()), offset);
             *(UInt16*)value = wVal;
-            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got word value %p", wVal);
+            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got word value 0x%x", wVal);
             break;
 
         case 32:
             lVal = OSReadLittleInt32((void*)(fMemMap->getVirtualAddress()), offset);
             *(UInt32*)value = lVal;
-            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got long value %p", lVal);
+            USBLog(1, "IOUSBControllerUserClient::ReadRegister - got long value 0x%lx", lVal);
             break;
 
         default:
@@ -364,9 +371,7 @@ IOUSBControllerUserClient::WriteRegister(UInt32 offset, UInt32 size, UInt32 valu
 void
 IOUSBControllerUserClient::stop( IOService * provider )
 {
-    IOLog("IOUSBControllerUserClient::stop\n");
-
-super::stop( provider );
+	super::stop( provider );
 
     if (fMemMap)
     {
@@ -381,7 +386,6 @@ IOUSBControllerUserClient::clientClose( void )
     /*
      * Kill ourselves off if the client closes or the client dies.
      */
-    IOLog("%s[%p]::clientClose isInactive = %d\n", getName(), this, isInactive());
     if( !isInactive())
         terminate();
 
