@@ -80,15 +80,55 @@ IOUSBControllerV2::init(OSDictionary * propTable)
     // allocate our expansion data
     if (!_v2ExpansionData)
     {
-	_v2ExpansionData = (V2ExpansionData *)IOMalloc(sizeof(V2ExpansionData));
-	if (!_v2ExpansionData)
-	    return false;
-	bzero(_v2ExpansionData, sizeof(V2ExpansionData));
+		_v2ExpansionData = (V2ExpansionData *)IOMalloc(sizeof(V2ExpansionData));
+		if (!_v2ExpansionData)
+			return false;
+		bzero(_v2ExpansionData, sizeof(V2ExpansionData));
     }
-	        
+	
     return (true);
 }
 
+
+bool 
+IOUSBControllerV2::start( IOService * provider )
+{
+    
+    if( !super::start(provider))
+        return (false);
+
+	// allocate a thread_call structure - code shared by EHCI and UHCI
+	_returnIsochDoneQueueThread = thread_call_allocate((thread_call_func_t)ReturnIsochDoneQueueEntry, (thread_call_param_t)this);
+	if ( !_returnIsochDoneQueueThread )
+	{
+		USBError(1, "IOUSBControllerV2[%p]::start - could not allocate thread callout function.",  this);
+		return false;
+	}
+	
+	return true;
+}
+
+
+
+void
+IOUSBControllerV2::free()
+{
+	
+    if (_returnIsochDoneQueueThread)
+    {
+        thread_call_cancel(_returnIsochDoneQueueThread);
+        thread_call_free(_returnIsochDoneQueueThread);
+    }
+	
+	// should I do an IOFree on the expansion data?
+	
+    super::free();
+}
+
+
+
+
+
 void
 IOUSBControllerV2::clearTTHandler( OSObject *	target,
                                    void *	parameter,
@@ -100,33 +140,33 @@ IOUSBControllerV2::clearTTHandler( OSObject *	target,
     UInt8		sent, back, todo;
     UInt8		hubAddr = command->GetAddress();
     
-    USBLog(5,"clearTTHandler: status (%lx)", status);
-
+    USBLog(5,"clearTTHandler: status (0x%x)", status);
+	
     sent = (command->GetStage() & 0x0f) << 4;
     back = command->GetStage() & 0xf0;
     todo = sent ^ back; /* thats xor */
-
+	
     if((todo & kSetupBack) != 0)
     {
-	USBLog(2,"clearTTHandler: Setup comming back to us, check and forget");
+		USBLog(2,"clearTTHandler: Setup comming back to us, check and forget");
         command->SetStage(command->GetStage() | kSetupBack);
     }
     else
     {
         command->SetStage(0);
-	USBLog(5, "%s[%p]::clearTTHandler - about to free IOUSBDevRequest (%p)", me->getName(), me, command->GetRequest());
-	IOFree(command->GetRequest(), sizeof(IOUSBDevRequest));
+		USBLog(5, "%s[%p]::clearTTHandler - about to free IOUSBDevRequest (%p)", me->getName(), me, command->GetRequest());
+		IOFree(command->GetRequest(), sizeof(IOUSBDevRequest));
 #if USEDYNAMICCOMMAND
-	USBLog(5,"clearTTHandler: We've already seen the setup, deallocate command (%p)", command);
-	me->_freeUSBCommandPool->returnCommand(command);   
+		USBLog(5,"clearTTHandler: We've already seen the setup, deallocate command (%p)", command);
+		me->_freeUSBCommandPool->returnCommand(command);   
 #else
-	USBLog(5,"clearTTHandler: We've already seen the setup, using static command");
+		USBLog(5,"clearTTHandler: We've already seen the setup, using static command");
 #endif
     }
     if (status != kIOReturnSuccess)
     {
-	USBLog(1, "%s[%p]::clearTTHandler - error response from hub, clearing hub endpoint stall", me->getName(), me);
-	me->UIMClearEndpointStall(hubAddr, 0, kUSBAnyDirn);
+		USBLog(1, "%s[%p]::clearTTHandler - error response from hub, clearing hub endpoint stall", me->getName(), me);
+		me->UIMClearEndpointStall(hubAddr, 0, kUSBAnyDirn);
     }
 }
 
@@ -142,109 +182,109 @@ IOUSBControllerV2::ClearTT(USBDeviceAddress fnAddress, UInt8 endpt, Boolean IN)
     IOUSBCompletion	completion;
     int 		i;
     IOReturn 		err;
-
+	
     USBLog(5,"+%s[%p]::ClearTT", getName(), this);
     hubAddress = _highSpeedHub[fnAddress];	// Address of its controlling hub.
     if(hubAddress == 0)	// Its not a high speed device, it doesn't need a clearTT
     {
-	USBLog(1,"-%s[%p]::ClearTT high speed device, returning", getName(), this);
-	return;
+		USBLog(1,"-%s[%p]::ClearTT high speed device, returning", getName(), this);
+		return;
     }
-
+	
 #if USEDYNAMICCOMMAND
     clearCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
     if ( clearCommand == NULL )
     {
-	IncreaseCommandPool();
-	
-	clearCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
-	if ( clearCommand == NULL )
-	{
-	    USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
-	    return;
-	}
+		IncreaseCommandPool();
+		
+		clearCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
+		if ( clearCommand == NULL )
+		{
+			USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
+			return;
+		}
     }
     USBLog(6, "%s[%p]::ClearTT V2 got command (%p)", getName(), this, clearCommand);
 #else
     clearCommand = _v2ExpansionData->ClearTTCommand;
     if ( clearCommand == NULL )
     {
-	_v2ExpansionData->ClearTTCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
-	clearCommand = _v2ExpansionData->ClearTTCommand;
-	if ( clearCommand == NULL )
-	{
-	    USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
-	    return;
-	}
-	USBLog(6,"%s[%p]::ClearTT gto static IOUSBCommand",getName(),this);
+		_v2ExpansionData->ClearTTCommand = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
+		clearCommand = _v2ExpansionData->ClearTTCommand;
+		if ( clearCommand == NULL )
+		{
+			USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBCommand",getName(),this);
+			return;
+		}
+		USBLog(6,"%s[%p]::ClearTT gto static IOUSBCommand",getName(),this);
     }
-
+	
     if(clearCommand->GetStage() != 0)
     {
-	USBLog(1,"%s[%p]::ClearTT our command is in use, giving up",getName(),this);
-	return;
+		USBLog(1,"%s[%p]::ClearTT our command is in use, giving up",getName(),this);
+		return;
     }
-
+	
 #endif
-
+	
     
     clearRequest = (IOUSBDevRequest*)IOMalloc(sizeof(IOUSBDevRequest));
     if (!clearRequest)
     {
-	USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBDevRequest", getName(), this);
+		USBLog(1,"%s[%p]::ClearTT Could not get a IOUSBDevRequest", getName(), this);
 #if USEDYNAMICCOMMAND
-	_freeUSBCommandPool->returnCommand(clearCommand);
+		_freeUSBCommandPool->returnCommand(clearCommand);
 #endif
-	return;
+		return;
     }
     USBLog(5, "%s[%p]::ClearTT - got IOUSBDevRequest (%p)", getName(), this, clearRequest);
-
+	
     wValue = (endpt & 0xf) | ( (fnAddress & 0x7f) << 4);
     if(IN)
     {
-	wValue  |= (1 << 15);
+		wValue  |= (1 << 15);
     }
     USBLog(5,"%s[%p]::ClearTT - V2 EP (%d) ADDR (%d) wValue (0x%x)", getName(), this, endpt, fnAddress, wValue);
-/*		
-3..0 Endpoint Number
-10..4 Device Address
-12..11 Endpoint Type	- Always controll == zero.
-14..13 Reserved, must be zero
-15 Direction, 1 = IN, 0 = OUT
-
-Endpoint Type
-00 Control
-01 Isochronous
-10 Bulk
-11 Interrupt
-
-*/
-
-/* Request details largely copied from AppleUSBHubPort::ClearTT */
-
+	/*		
+		3..0 Endpoint Number
+		10..4 Device Address
+		12..11 Endpoint Type	- Always controll == zero.
+		14..13 Reserved, must be zero
+		15 Direction, 1 = IN, 0 = OUT
+		
+		Endpoint Type
+		00 Control
+		01 Isochronous
+		10 Bulk
+		11 Interrupt
+		
+		*/
+	
+	/* Request details largely copied from AppleUSBHubPort::ClearTT */
+	
     clearRequest->bmRequestType = 0x23;
     clearRequest->bRequest = 8;
     clearRequest->wValue = wValue;
     if(_v2ExpansionData->_multiTT[hubAddress])
     {  // MultiTT hub needs port address here
-	clearRequest->wIndex = _highSpeedPort[fnAddress];
+		clearRequest->wIndex = _highSpeedPort[fnAddress];
     }
     else
     {  // Single TT hubs need 1 here
-	clearRequest->wIndex = 1;
+		clearRequest->wIndex = 1;
     }
     clearRequest->wLength = 0;
     clearRequest->pData = NULL;
     clearRequest->wLenDone = 0;
-
-/* This copies large parts of IOUSBController::DeviceRequest, its not using IOUSBController::DeviceRequest */
-/* Because we're already inside the lock and don't want to go through the gate again. */
-
+	
+	/* This copies large parts of IOUSBController::DeviceRequest, its not using IOUSBController::DeviceRequest */
+	/* Because we're already inside the lock and don't want to go through the gate again. */
+	
     completion.target    = (void *)this;
     completion.action    = (IOUSBCompletionAction) &clearTTHandler;
     completion.parameter = clearCommand;
     clearCommand->SetUSLCompletion(completion);
-
+	
     clearCommand->SetUseTimeStamp(false);
     clearCommand->SetSelector(DEVICE_REQUEST);
     clearCommand->SetRequest(clearRequest);
@@ -258,82 +298,82 @@ Endpoint Type
     clearCommand->SetStage(0);
     
     for (i=0; i < 10; i++)
-	clearCommand->SetUIMScratch(i, 0);
-
+		clearCommand->SetUIMScratch(i, 0);
+	
     err = ControlTransaction(clearCommand);	// Wait for completion? Or just fire and forget?
     if (err)
     {
-	USBLog(1, "%s[%p]::ClearTT - error %p returned from ControlTransaction", getName(), this, err);
+		USBLog(1, "%s[%p]::ClearTT - error 0x%x returned from ControlTransaction", getName(), this, err);
     }
 }
 
 
 
 IOReturn IOUSBControllerV2::OpenPipe(USBDeviceAddress address, UInt8 speed,
-						Endpoint *endpoint)
+									 Endpoint *endpoint)
 {
     return _commandGate->runAction(DoCreateEP, (void *)(UInt32)address,
-			(void *)(UInt32)speed, endpoint);
+								   (void *)(UInt32)speed, endpoint);
 }
 
 
 IOReturn 
 IOUSBControllerV2::DoCreateEP(OSObject *owner,
-                           void *arg0, void *arg1,
-                           void *arg2, void *arg3)
+							  void *arg0, void *arg1,
+							  void *arg2, void *arg3)
 {
     IOUSBControllerV2 *me = (IOUSBControllerV2 *)owner;
     UInt8 address = (UInt8)(UInt32)arg0;
     UInt8 speed = (UInt8)(UInt32)arg1;
     Endpoint *endpoint = (Endpoint *)arg2;
     IOReturn err;
-
+	
     USBLog(5,"%s[%p]::DoCreateEP, high speed ancestor hub:%d, port:%d", me->getName(), me, me->_highSpeedHub[address], me->_highSpeedPort[address]);
-
+	
     switch (endpoint->transferType)
     {
         case kUSBInterrupt:
             err = me->UIMCreateInterruptEndpoint(address,
-                                             endpoint->number,
-                                             endpoint->direction,
-                                             speed,
-                                             endpoint->maxPacketSize,
-                                             endpoint->interval,
-                                             me->_highSpeedHub[address],
-                                             me->_highSpeedPort[address]);
+												 endpoint->number,
+												 endpoint->direction,
+												 speed,
+												 endpoint->maxPacketSize,
+												 endpoint->interval,
+												 me->_highSpeedHub[address],
+												 me->_highSpeedPort[address]);
             break;
-
+			
         case kUSBBulk:
             err = me->UIMCreateBulkEndpoint(address,
-                                        endpoint->number,
-                                        endpoint->direction,
-                                        speed,
-                                        endpoint->maxPacketSize,
-                                        me->_highSpeedHub[address],
-                                        me->_highSpeedPort[address]);
+											endpoint->number,
+											endpoint->direction,
+											speed,
+											endpoint->maxPacketSize,
+											me->_highSpeedHub[address],
+											me->_highSpeedPort[address]);
             break;
-
+			
         case kUSBControl:
             err = me->UIMCreateControlEndpoint(address,
-                                           endpoint->number,
-                                           endpoint->maxPacketSize,
-                                           speed,
-                                           me->_highSpeedHub[address],
-                                           me->_highSpeedPort[address]);
+											   endpoint->number,
+											   endpoint->maxPacketSize,
+											   speed,
+											   me->_highSpeedHub[address],
+											   me->_highSpeedPort[address]);
             break;
-
+			
         case kUSBIsoc:
-	    USBLog(4, "%s[%p]::DoCreateEP - Creating Isoch EP with interval %d [raw %d]", me->getName(), me, (endpoint->interval == 4) ? 8 : 1, endpoint->interval);
-	    // only support intervals of 1 and 8
+			USBLog(4, "%s[%p]::DoCreateEP - Creating Isoch EP with interval %d [raw %d]", me->getName(), me, (endpoint->interval == 4) ? 8 : 1, endpoint->interval);
+			// only support intervals of 1 and 8
             err = me->UIMCreateIsochEndpoint(address,
-                                        endpoint->number,
-                                        endpoint->maxPacketSize,
-                                        endpoint->direction,
-                                        me->_highSpeedHub[address],
-                                        me->_highSpeedPort[address],
-					(endpoint->interval == 4) ? 8 : 1);
+											 endpoint->number,
+											 endpoint->maxPacketSize,
+											 endpoint->direction,
+											 me->_highSpeedHub[address],
+											 me->_highSpeedPort[address],
+											 (endpoint->interval == 4) ? 8 : 1);
             break;
-
+			
         default:
             err = kIOReturnBadArgument;
             break;
@@ -344,12 +384,12 @@ IOUSBControllerV2::DoCreateEP(OSObject *owner,
 
 IOReturn 
 IOUSBControllerV2::CreateDevice(	IOUSBDevice 		*newDevice,
-                                USBDeviceAddress	deviceAddress,
-                                UInt8		 	maxPacketSize,
-                                UInt8			speed,
-                                UInt32			powerAvailable,
-                                USBDeviceAddress		hub,
-                                int      port)
+									USBDeviceAddress	deviceAddress,
+									UInt8		 	maxPacketSize,
+									UInt8			speed,
+									UInt32			powerAvailable,
+									USBDeviceAddress		hub,
+									int      port)
 {
     USBLog(5,"%s[%p]::CreateDevice, new method called with hub:%d, port:%d", getName(), this, hub, port);
     
@@ -371,7 +411,7 @@ IOUSBControllerV2::CreateDevice(	IOUSBDevice 		*newDevice,
         _highSpeedHub[deviceAddress] = 0;
         _highSpeedPort[deviceAddress] = 0;
     }
-
+	
     USBLog(5,"%s[%p]::CreateDevice, high speed ancestor hub:%d, port:%d",getName(), this, _highSpeedHub[deviceAddress], _highSpeedPort[deviceAddress]);
     
     return (super::CreateDevice(newDevice, deviceAddress, maxPacketSize, speed, powerAvailable));
@@ -381,7 +421,7 @@ IOReturn
 IOUSBControllerV2::ConfigureDeviceZero(UInt8 maxPacketSize, UInt8 speed, USBDeviceAddress hub, int port)
 {
     USBLog(5,"%s[%p]::ConfigureDeviceZero, new method called with hub:%d, port:%d", getName(), this, hub, port);
-
+	
     if(speed != kUSBDeviceSpeedHigh)
     {
         if(_highSpeedHub[hub] == 0)	// this is the first non high speed device in this chain
@@ -400,7 +440,7 @@ IOUSBControllerV2::ConfigureDeviceZero(UInt8 maxPacketSize, UInt8 speed, USBDevi
         _highSpeedHub[0] = 0;
         _highSpeedPort[0] = 0;
     }
-
+	
     USBLog(5, "%s[%p]::CreateDevice, high speed ancestor hub:%d, port:%d", getName(), this, _highSpeedHub[0], _highSpeedPort[0]);
     
     return (super::ConfigureDeviceZero(maxPacketSize, speed));
@@ -417,13 +457,13 @@ IOUSBControllerV2::DOHSHubMaintenance(OSObject *owner, void *arg0, void *arg1, v
     UInt32 command = (UInt32)arg1;
     UInt32 flags = (UInt32)arg2;
     UInt8 multi;
-
-    USBLog(5,"%s[%p]::DOHSHubMaintenance, command: %d, flags: %d", me->getName(), me, command, flags);
-
+	
+    USBLog(5,"%s[%p]::DOHSHubMaintenance, command: %ld, flags: %ld", me->getName(), me, command, flags);
+	
     multi = ((flags & kUSBHSHubFlagsMultiTT) != 0);
     me->_v2ExpansionData->_multiTT[highSpeedHub] = multi;
     USBLog(3,"%s[%p]::DOHSHubMaintenance hub at %d is multiTT:%d", me->getName(), me, highSpeedHub, me->_v2ExpansionData->_multiTT[highSpeedHub]);
-
+	
     return me->UIMHubMaintenance(highSpeedHub, 0, command, flags);
 }
 
@@ -435,9 +475,9 @@ IOUSBControllerV2::DOSetTestMode(OSObject *owner, void *arg0, void *arg1, void *
     IOUSBControllerV2 *me = (IOUSBControllerV2 *)owner;
     UInt32 mode = (UInt32)arg0;
     UInt32 port = (UInt32)arg1;
-
-    USBLog(5,"%s[%p]::DOSetTestMode, mode: %d, port: %d", me->getName(), me, mode, port);
-
+	
+    USBLog(5,"%s[%p]::DOSetTestMode, mode: %ld, port: %ld", me->getName(), me, mode, port);
+	
     return me->UIMSetTestMode(mode, port);
 }
 
@@ -448,7 +488,7 @@ IOReturn
 IOUSBControllerV2::AddHSHub(USBDeviceAddress highSpeedHub, UInt32 flags)
 {
     return _commandGate->runAction(DOHSHubMaintenance, (void *)(UInt32)highSpeedHub,
-			(void *)(UInt32)kUSBHSHubCommandAddHub, (void *)flags);
+								   (void *)(UInt32)kUSBHSHubCommandAddHub, (void *)flags);
 }
 
 
@@ -467,10 +507,10 @@ IOReturn
 IOUSBControllerV2::RemoveHSHub(USBDeviceAddress highSpeedHub)
 {
     return _commandGate->runAction(DOHSHubMaintenance, (void *)(UInt32)highSpeedHub,
-			(void *)(UInt32)kUSBHSHubCommandRemoveHub, NULL);
+								   (void *)(UInt32)kUSBHSHubCommandRemoveHub, NULL);
 }
 
-    
+
 
 OSMetaClassDefineReservedUsed(IOUSBControllerV2,  3);
 IOReturn		
@@ -503,36 +543,36 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
     IOUSBCompletion 	theCompletion;
     IOUSBCompletionAction 	theAction;
     int			i;
-
-    USBLog(7, "%s[%p]::ReadV2 - reqCount = %d", getName(), this, reqCount);
-
+	
+    USBLog(7, "%s[%p]::ReadV2 - reqCount = %ld", getName(), this, reqCount);
+	
     // Validate its a inny pipe and that there is a buffer
     if ((endpoint->direction != kUSBIn) || !buffer || (buffer->getLength() < reqCount))
     {
         USBLog(5, "%s[%p]::ReadV2 - direction is not kUSBIn (%d), No Buffer, or buffer length < reqCount (%ld < %ld). Returning kIOReturnBadArgument(0x%x)", getName(), this, endpoint->direction,  buffer->getLength(), reqCount, kIOReturnBadArgument);
         return kIOReturnBadArgument;
     }
-
+	
     if ((endpoint->transferType != kUSBBulk) && (noDataTimeout || completionTimeout))
     {
         USBLog(5, "%s[%p]::ReadV2 - Pipe is NOT kUSBBulk (%d) AND specified a timeout (%ld, %ld).  Returning kIOReturnBadArgument(0x%x)", getName(), this, endpoint->transferType, noDataTimeout, completionTimeout, kIOReturnBadArgument);
         return kIOReturnBadArgument;							// timeouts only on bulk pipes
     }
-
+	
     // Validate the completion
     if (!completion)
     {
         USBLog(5, "%s[%p]::ReadV2 - No Completion routine.  Returning kIOReturnNoCompletion(0x%x)", getName(), this, kIOReturnNoCompletion);
         return kIOReturnNoCompletion;
     }
-
+	
     // Validate the command gate
     if (!_commandGate)
     {
         USBLog(5, "%s[%p]::ReadV2 - Could not get _commandGate.  Returning kIOReturnInternalError(0x%x)", getName(), this, kIOReturnInternalError);
         return kIOReturnInternalError;
     }
-
+	
     // allocate the command
     command = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
 	
@@ -541,7 +581,7 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
     if ( command == NULL )
     {
         IncreaseCommandPool();
-
+		
         command = (IOUSBCommand *)_freeUSBCommandPool->getCommand(false);
         if ( command == NULL )
         {
@@ -549,18 +589,18 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
             return kIOReturnNoResources;
         }
     }
-
+	
     theCompletion.target = completion->target;
     theCompletion.action = (IOUSBCompletionAction)completion->action;
     theCompletion.parameter = completion->parameter;
-
+	
 	// Set up a flag indicating that we have a synchronous request in this command
 	//
     if (  (UInt32) completion->action == (UInt32) &IOUSBSyncCompletion )
 		command->SetIsSyncTransfer(true);
 	else
 		command->SetIsSyncTransfer(false);
-
+	
     command->SetUseTimeStamp(true);
     command->SetSelector(READ);
     command->SetRequest(0);            	// Not a device request
@@ -575,17 +615,17 @@ IOUSBControllerV2::ReadV2(IOMemoryDescriptor *buffer, USBDeviceAddress address,
     command->SetCompletionTimeout(completionTimeout);
     for (i=0; i < 10; i++)
         command->SetUIMScratch(i, 0);
-
+	
     nullCompletion.target = (void *) NULL;
     nullCompletion.action = (IOUSBCompletionAction) NULL;
     nullCompletion.parameter = (void *) NULL;
     command->SetDisjointCompletion(nullCompletion);
-
+	
     err = CheckForDisjointDescriptor(command, endpoint->maxPacketSize);
     if (kIOReturnSuccess == err)
 	{
         err = _commandGate->runAction(DoIOTransfer, command);
-
+		
 		// If we have a sync request, then we always return the command after the DoIOTransfer.  If it's an async request, we only return it if 
 		// we get an immediate error
 		//
@@ -618,17 +658,331 @@ IOUSBControllerV2::UIMCreateIsochEndpoint(short functionAddress, short endpointN
 
 
 
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  9);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  10);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  11);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  12);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  13);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  14);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  15);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  16);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  17);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  18);
-OSMetaClassDefineReservedUnused(IOUSBControllerV2,  19);
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  9);
+IOUSBControllerIsochEndpoint*
+IOUSBControllerV2::AllocateIsochEP()
+{
+	USBError(1, "IOUSBControllerV2[%p]::AllocateIsochEP - should be overriden in a subclass", this);
+	return NULL;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  10);
+IOReturn
+IOUSBControllerV2::DeallocateIsochEP(IOUSBControllerIsochEndpoint* pEP)
+{
+	USBLog(4, "%s[%p]::DeallocateIsochEP (%p)",getName(), this, pEP);
+    pEP->nextEP = _freeIsochEPList;
+    _freeIsochEPList = pEP;
+    return kIOReturnSuccess;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  11);
+IOUSBControllerIsochEndpoint*
+IOUSBControllerV2::FindIsochronousEndpoint(short								functionAddress,
+										   short								endpointNumber,
+										   short								direction,
+										   IOUSBControllerIsochEndpoint			**ppEPBack)
+{
+    IOUSBControllerIsochEndpoint		*pEP, *pBack;
+    
+    pEP = _isochEPList;
+    pBack = NULL;
+    while (pEP)
+    {
+		if ((pEP->functionAddress == functionAddress)
+			&& (pEP->endpointNumber == endpointNumber)
+			&& (pEP->direction == direction))
+			break;
+		pBack = pEP;
+		pEP = pEP->nextEP;
+    }
+    if (pEP && ppEPBack)
+		*ppEPBack = pBack;
+    return pEP;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  12);
+IOUSBControllerIsochEndpoint*
+IOUSBControllerV2::CreateIsochronousEndpoint(short					functionAddress,
+											 short					endpointNumber,
+											 short 					direction)
+{
+    IOUSBControllerIsochEndpoint*			pEP;
+	int										i;
+    
+    USBLog(4, "+%s[%p]::CreateIsochronousEndpoint (%d:%d:%d)", getName(), this, functionAddress, endpointNumber, direction);
+	pEP = _freeIsochEPList;
+	if (!pEP)
+	{
+		pEP = AllocateIsochEP();
+		USBLog(4, "%s[%p]::CreateIsochronousEndpoint  called AllocateIsochEP (%p)",getName(), this, pEP);
+	}
+	if (pEP)
+	{
+		_freeIsochEPList = pEP->nextEP;									// unlink from free list
+		pEP->init();													// make sure to reinitialize it
+		pEP->nextEP = _isochEPList;
+		_isochEPList = pEP;
+		pEP->functionAddress = functionAddress;
+		pEP->endpointNumber = endpointNumber;
+		pEP->direction = direction;
+	}
+    return pEP;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  13);
+void
+IOUSBControllerV2::PutTDonToDoList(IOUSBControllerIsochEndpoint* pED, IOUSBControllerIsochListElement *pTD)
+{
+    USBLog(7, "AppleUSBEHCI[%p]::PutTDonToDoList - pED (%p) pTD (%p) frameNumber(%Lx)", this, pED, pTD, pTD->_frameNumber);
+    // Link TD into todo list
+    if(pED->toDoList == NULL)
+    {
+		// as the head of a new list
+		pED->toDoList = pTD;
+    }
+    else
+    {
+		// at the tail of the old list
+		pED->toDoEnd->_logicalNext = pTD;
+    }
+    // no matter what we are the new tail
+    pED->toDoEnd = pTD;
+	pED->onToDoList++;
+    pED->activeTDs++;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  14);
+IOUSBControllerIsochListElement *
+IOUSBControllerV2::GetTDfromToDoList(IOUSBControllerIsochEndpoint* pED)
+{
+    IOUSBControllerIsochListElement	*pTD;
+    
+	// Do not call USBLog here, as this can be called from AddIsocFramesToSchedule, which holds off preemption
+    pTD = pED->toDoList;
+    if (pTD)
+    {
+		if (pTD == pED->toDoEnd)
+			pED->toDoList = pED->toDoEnd = NULL;
+		else
+			pED->toDoList = OSDynamicCast(IOUSBControllerIsochListElement, pTD->_logicalNext);
+		// USBLog(7, "AppleUSBEHCI[%p]::GetTDfromToDoList - pED (%p) pTD (%p) frameNumber(%Lx)", this, pED, pTD, pTD->_frameNumber);
+		pED->onToDoList--;
+    }
+    return pTD;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  15);
+void
+IOUSBControllerV2::PutTDonDoneQueue(IOUSBControllerIsochEndpoint* pED, IOUSBControllerIsochListElement *pTD, bool checkDeferred)
+{
+    IOUSBControllerIsochListElement	*deferredTD;
+	
+	// Do not call USBLog here, as this can be called from AddIsocFramesToSchedule, which holds off preemption
+    if (checkDeferred)
+    {
+		while (pED->deferredQueue && (pED->deferredQueue->_frameNumber < pTD->_frameNumber))
+		{
+			deferredTD = GetTDfromDeferredQueue(pED);
+			PutTDonDoneQueue(pED, deferredTD, false);
+		}
+    }
+    
+    if(pED->doneQueue == NULL)
+    {
+		// as the head of a new list
+		pED->doneQueue = pTD;
+    }
+    else
+    {
+		// at the tail of the old list
+		pED->doneEnd->_logicalNext = pTD;
+    }
+    // and not matter what we are now the new tail
+    pED->doneEnd = pTD;
+	pED->onDoneQueue++;
+	
+	// if there are no TDs on the schedule, and no TDs on the toDO list, then we should clear out the deferred queue
+	if (checkDeferred && !pED->scheduledTDs && !pED->toDoList)
+	{
+		deferredTD = GetTDfromDeferredQueue(pED);
+		while (deferredTD)
+		{
+			PutTDonDoneQueue(pED, deferredTD, false);
+			deferredTD = GetTDfromDeferredQueue(pED);
+		}
+	}
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  16);
+IOUSBControllerIsochListElement *
+IOUSBControllerV2::GetTDfromDoneQueue(IOUSBControllerIsochEndpoint* pED)
+{
+    IOUSBControllerIsochListElement	*pTD;
+    
+    pTD = pED->doneQueue;
+    if (pTD)
+    {
+		if (pTD == pED->doneEnd)
+			pED->doneQueue = pED->doneEnd = NULL;
+		else
+			pED->doneQueue = OSDynamicCast(IOUSBControllerIsochListElement, pTD->_logicalNext);
+		pED->onDoneQueue--;
+		pED->activeTDs--;
+    }
+    return pTD;
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  17);
+void
+IOUSBControllerV2::PutTDonDeferredQueue(IOUSBControllerIsochEndpoint* pED, IOUSBControllerIsochListElement *pTD)
+{
+	// Do not call USBLog here, as this can be called from AddIsocFramesToSchedule, which holds off preemption
+	// USBLog(7, "AppleUSBEHCI[%p]::PutTDonDeferredQueue(%p, %p)", this, pED, pTD);
+	
+    if(pED->deferredQueue == NULL)
+    {
+		// as the head of a new list
+		pED->deferredQueue = pTD;
+    }
+    else
+    {
+		// at the tail of the old list
+		pED->deferredEnd->_logicalNext = pTD;
+    }
+    // and not matter what we are now the new tail
+    pED->deferredEnd = pTD;
+	pED->deferredTDs++;
+	
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  18);
+IOUSBControllerIsochListElement *
+IOUSBControllerV2::GetTDfromDeferredQueue(IOUSBControllerIsochEndpoint* pED)
+{
+    IOUSBControllerIsochListElement	*pTD;
+    
+    pTD = pED->deferredQueue;
+    if (pTD)
+    {
+		if (pTD == pED->deferredEnd)
+			pED->deferredQueue = pED->deferredEnd = NULL;
+		else
+			pED->deferredQueue = OSDynamicCast(IOUSBControllerIsochListElement, pTD->_logicalNext);
+		pED->deferredTDs--;
+    }
+	USBLog(7, "AppleUSBEHCI[%p]::GetTDfromDeferredQueue(%p) returning %p", this, pED, pTD);
+    return pTD;
+}
+
+
+// this is a static method - hence no slot
+void
+IOUSBControllerV2::ReturnIsochDoneQueueEntry(OSObject *target, thread_call_param_t endpointPtr)
+{
+    IOUSBControllerV2 *					me = OSDynamicCast(IOUSBControllerV2, target);
+    IOUSBControllerIsochEndpoint* 	pEP = (IOUSBControllerIsochEndpoint*) endpointPtr;
+	
+    if (!me || !pEP)
+        return;
+	
+    me->retain();
+    me->ReturnIsochDoneQueue(pEP);
+    me->release();
+}
+
+
+
+OSMetaClassDefineReservedUsed(IOUSBControllerV2,  19);
+void 
+IOUSBControllerV2::ReturnIsochDoneQueue(IOUSBControllerIsochEndpoint* pEP)
+{
+    IOUSBControllerIsochListElement		*pTD = GetTDfromDoneQueue(pEP);
+    IOUSBIsocFrame						*pFrames = NULL;
+		
+    USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue (%p)", this, pEP);
+    if (pTD)
+    {
+		pFrames = pTD->_pFrames;
+    }
+	else
+	{
+		USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - no TDs to return", this);
+	}
+    while(pTD)
+    {
+		USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue: TD %p", this, pTD);
+		if( pTD->_completion.action != NULL)
+		{
+			IOUSBIsocCompletionAction 	pHandler;
+			
+			pHandler = pTD->_completion.action;
+			pTD->_completion.action = NULL;
+			if (pEP->accumulatedStatus == kIOUSBBufferUnderrunErr)
+			{
+				USBLog(1, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - kIOReturnBufferUnderrunErr (PCI issue perhaps)", this);
+			}
+			if (pEP->accumulatedStatus == kIOUSBBufferOverrunErr)
+			{
+				USBLog(1, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - kIOReturnBufferOverrunErr (PCI issue perhaps)", this);
+			}
+			if ((pEP->accumulatedStatus == kIOReturnOverrun) && (pEP->direction == kUSBIn))
+			{
+				USBLog(1, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - kIOReturnOverrun on IN - device babbling?", this);
+			}
+			USBLog(7, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue- calling handler(target: %p, comp.param: %p, status: %p, pFrames: %p)", this,
+				   pTD->_completion.target, pTD->_completion.parameter, (void*)pEP->accumulatedStatus, pFrames);
+			(*pHandler) (pTD->_completion.target,  pTD->_completion.parameter, pEP->accumulatedStatus, pFrames);
+			
+			_activeIsochTransfers--;
+			if ( _activeIsochTransfers < 0 )
+			{
+				USBLog(1, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - _activeIsochTransfers went negative (%d).  We lost one somewhere", this, (int)_activeIsochTransfers);
+			}
+			
+			// if the accumulated status is aborted, then we need to keep that status until we are done
+			// otherwise the status will be in the endpoint when we get to the callback case and will
+			// be reset afterwards
+			if (pEP->accumulatedStatus != kIOReturnAborted)
+			{
+				if (pEP->accumulatedStatus != kIOReturnSuccess && (pEP->accumulatedStatus != kIOReturnUnderrun) )
+					USBLog(6, "IOUSBControllerV2[%p]::ReturnIsocDoneQueue - resetting status from 0x%x", this, pEP->accumulatedStatus);
+				pEP->accumulatedStatus = kIOReturnSuccess;
+			}
+			pTD->Deallocate(this);
+			pTD = GetTDfromDoneQueue(pEP);
+			if (pTD)
+				pFrames = pTD->_pFrames;
+		}
+		else
+		{
+			pTD->Deallocate(this);
+			pTD = GetTDfromDoneQueue(pEP);
+		}
+    }
+    
+}
+
+
+
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  20);
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  21);
 OSMetaClassDefineReservedUnused(IOUSBControllerV2,  22);
